@@ -186,8 +186,9 @@ static sw_inline char* sw_strdup(const char *s)
 }
 static sw_inline char* sw_strndup(const char *s, size_t n)
 {
-    char *p = sw_malloc(n);
+    char *p = sw_malloc(n + 1);
     strncpy(p, s, n);
+    p[n] = '\0';
     return p;
 }
 #else
@@ -554,6 +555,8 @@ typedef struct _swConnection
      */
     time_t last_time;
 
+    uint16_t timewheel_index;
+
     /**
      * bind uid
      */
@@ -719,6 +722,22 @@ typedef struct _swSendData
     char *data;
 } swSendData;
 
+typedef struct
+{
+    off_t offset;
+    size_t length;
+    char filename[0];
+} swSendFile_request;
+
+//------------------TimeWheel--------------------
+typedef struct
+{
+    uint16_t current;
+    uint16_t size;
+    swHashMap **wheel;
+
+} swTimeWheel;
+
 typedef void * (*swThreadStartFunc)(void *);
 typedef int (*swHandle)(swEventData *buf);
 typedef void (*swSignalHander)(int);
@@ -1211,6 +1230,8 @@ void swoole_update_time(void);
 double swoole_microtime(void);
 void swoole_rtrim(char *str, int len);
 void swoole_redirect_stdout(int new_fd);
+int swoole_add_function(const char *name, void* func);
+void* swoole_get_function(char *name, uint32_t length);
 
 static sw_inline uint64_t swoole_hton64(uint64_t host)
 {
@@ -1252,7 +1273,7 @@ int swSocket_sendto_blocking(int fd, void *__buf, size_t __n, int flag, struct s
 int swSocket_set_buffer_size(int fd, int buffer_size);
 int swSocket_udp_sendto(int server_sock, char *dst_ip, int dst_port, char *data, uint32_t len);
 int swSocket_udp_sendto6(int server_sock, char *dst_ip, int dst_port, char *data, uint32_t len);
-int swSocket_sendfile_sync(int sock, char *filename, off_t offset, double timeout);
+int swSocket_sendfile_sync(int sock, char *filename, off_t offset, size_t length, double timeout);
 int swSocket_write_blocking(int __fd, void *__data, int __len);
 
 static sw_inline int swWaitpid(pid_t __pid, int *__stat_loc, int __options)
@@ -1359,6 +1380,16 @@ struct _swReactor
 
     uint32_t max_socket;
 
+#ifdef SW_USE_MALLOC_TRIM
+    time_t last_mallc_trim_time;
+#endif
+
+#ifdef SW_USE_TIMEWHEEL
+    swTimeWheel *timewheel;
+    uint16_t heartbeat_interval;
+    time_t last_heartbeat_time;
+#endif
+
     /**
      * for thread
      */
@@ -1718,17 +1749,23 @@ int swProtocol_recv_check_length(swProtocol *protocol, swConnection *conn, swStr
 int swProtocol_recv_check_eof(swProtocol *protocol, swConnection *conn, swString *buffer);
 
 //--------------------------------timer------------------------------
-typedef struct _swTimer_node
+typedef struct _swTimer swTimer;
+typedef struct _swTimer_node swTimer_node;
+
+typedef void (*swTimerCallback)(swTimer *, swTimer_node *);
+
+struct _swTimer_node
 {
     swHeap_node *heap_node;
     void *data;
+    swTimerCallback callback;
     int64_t exec_msec;
     uint32_t interval;
     long id;
-    uint8_t remove :1;
-} swTimer_node;
+    uint8_t remove;
+};
 
-typedef struct _swTimer
+struct _swTimer
 {
     /*--------------timerfd & signal timer--------------*/
     swHeap *heap;
@@ -1743,22 +1780,27 @@ typedef struct _swTimer
     /*-----------------for EventTimer-------------------*/
     struct timeval basetime;
     /*--------------------------------------------------*/
-    int (*set)(struct _swTimer *timer, long exec_msec);
-    /*-----------------event callback-------------------*/
-    void (*onAfter)(struct _swTimer *timer, swTimer_node *event);
-    void (*onTick)(struct _swTimer *timer, swTimer_node *event);
-} swTimer;
+    int (*set)(swTimer *timer, long exec_msec);
+    swTimer_node* (*add)(swTimer *timer, int _msec, int persistent, void *data, swTimerCallback callback);
+};
 
 int swTimer_init(long msec);
-swTimer_node* swTimer_add(swTimer *timer, int _msec, int interval, void *data);
 swTimer_node* swTimer_get(swTimer *timer, long id);
-void swTimer_del(swTimer *timer, swTimer_node *node);
+int swTimer_del(swTimer *timer, swTimer_node *node);
 void swTimer_free(swTimer *timer);
 int swTimer_select(swTimer *timer);
 
 int swSystemTimer_init(int msec, int use_pipe);
 void swSystemTimer_signal_handler(int sig);
 int swSystemTimer_event_handler(swReactor *reactor, swEvent *event);
+
+swTimeWheel* swTimeWheel_new(uint16_t size);
+void swTimeWheel_free(swTimeWheel *tw);
+void swTimeWheel_forward(swTimeWheel *tw, swReactor *reactor);
+void swTimeWheel_add(swTimeWheel *tw, swConnection *conn);
+void swTimeWheel_update(swTimeWheel *tw, swConnection *conn);
+void swTimeWheel_remove(swTimeWheel *tw, swConnection *conn);
+#define swTimeWheel_new_index(tw)   (tw->current == 0 ? tw->size - 1 : tw->current - 1)
 //--------------------------------------------------------------
 //Share Memory
 typedef struct
