@@ -140,6 +140,30 @@ static inline int SW_Z_TYPE_P(zval *z)
 #define IS_TRUE    1
 inline int SW_Z_TYPE_P(zval *z);
 #define SW_Z_TYPE_PP(z)        SW_Z_TYPE_P(*z)
+
+static inline char* sw_http_build_query(zval *data, zend_size_t *length, smart_str *formstr TSRMLS_DC)
+{
+#if PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION == 3
+    if (php_url_encode_hash_ex(HASH_OF(data), formstr, NULL, 0, NULL, 0, NULL, 0, NULL, NULL TSRMLS_CC) == FAILURE)
+#else
+    if (php_url_encode_hash_ex(HASH_OF(data), formstr, NULL, 0, NULL, 0, NULL, 0, NULL, NULL, (int) PHP_QUERY_RFC1738 TSRMLS_CC) == FAILURE)
+#endif
+    {
+        if (formstr->c)
+        {
+            smart_str_free(formstr);
+        }
+        return NULL;
+    }
+    if (!formstr->c)
+    {
+        return NULL;
+    }
+    smart_str_0(formstr);
+    *length = formstr->len;
+    return formstr->c;
+}
+
 #else
 #define sw_php_var_serialize                php_var_serialize
 typedef size_t zend_size_t;
@@ -362,6 +386,26 @@ static inline int sw_zend_hash_exists(HashTable *ht, char *k, int len)
         return SUCCESS;
     }
 }
+
+static inline char* sw_http_build_query(zval *data, zend_size_t *length, smart_str *formstr TSRMLS_DC)
+{
+    if (php_url_encode_hash_ex(HASH_OF(data), formstr, NULL, 0, NULL, 0, NULL, 0, NULL, NULL, (int) PHP_QUERY_RFC1738) == FAILURE)
+    {
+        if (formstr->s)
+        {
+            smart_str_free(formstr);
+        }
+        return NULL;
+    }
+    if (!formstr->s)
+    {
+        return NULL;
+    }
+    smart_str_0(formstr);
+    *length = formstr->s->len;
+    return formstr->s->val;
+}
+
 #endif
 
 #endif /* EXT_SWOOLE_PHP7_WRAPPER_H_ */
