@@ -46,7 +46,7 @@
 #include "Client.h"
 #include "async.h"
 
-#define PHP_SWOOLE_VERSION  "1.9.10"
+#define PHP_SWOOLE_VERSION  "1.9.11"
 #define PHP_SWOOLE_CHECK_CALLBACK
 
 /**
@@ -323,7 +323,6 @@ PHP_FUNCTION(swoole_timer_clear);
 //---------------------------------------------------------
 //                  other
 //---------------------------------------------------------
-PHP_FUNCTION(swoole_load_module);
 PHP_FUNCTION(swoole_strerror);
 PHP_FUNCTION(swoole_errno);
 //---------------------------------------------------------
@@ -354,7 +353,6 @@ void swoole_http2_client_init(int module_number TSRMLS_DC);
 void swoole_websocket_init(int module_number TSRMLS_DC);
 void swoole_buffer_init(int module_number TSRMLS_DC);
 void swoole_mysql_init(int module_number TSRMLS_DC);
-void swoole_module_init(int module_number TSRMLS_DC);
 void swoole_mmap_init(int module_number TSRMLS_DC);
 void swoole_channel_init(int module_number TSRMLS_DC);
 #if PHP_MAJOR_VERSION == 7
@@ -461,7 +459,6 @@ ZEND_BEGIN_MODULE_GLOBALS(swoole)
     zend_bool use_namespace;
     zend_bool fast_serialize;
     long socket_buffer_size;
-    char *modules;
 ZEND_END_MODULE_GLOBALS(swoole)
 
 extern ZEND_DECLARE_MODULE_GLOBALS(swoole);
