@@ -757,14 +757,14 @@ void swoole_fcntl_set_option(int sock, int nonblock, int cloexec)
         opts = opts & ~O_NONBLOCK;
     }
 
-#ifdef O_CLOEXEC
+#ifdef FD_CLOEXEC
     if (cloexec)
     {
-        opts = opts | O_CLOEXEC;
+        opts = opts | FD_CLOEXEC;
     }
     else
     {
-        opts = opts & ~O_CLOEXEC;
+        opts = opts & ~FD_CLOEXEC;
     }
 #endif
 
@@ -943,6 +943,33 @@ int swoole_gethostbyname(int flags, char *name, char *addr)
     return SW_OK;
 }
 
+int swoole_add_function(const char *name, void* func)
+{
+    if (SwooleG.functions == NULL)
+    {
+        SwooleG.functions = swHashMap_new(64, NULL);
+        if (SwooleG.functions == NULL)
+        {
+            return SW_ERR;
+        }
+    }
+    if (swHashMap_find(SwooleG.functions, (char *) name, strlen(name)) != NULL)
+    {
+        swWarn("Function '%s' already exists.", name);
+        return SW_ERR;
+    }
+    return swHashMap_add(SwooleG.functions, (char *) name, strlen(name), func);
+}
+
+void* swoole_get_function(char *name, uint32_t length)
+{
+    if (!SwooleG.functions)
+    {
+        return NULL;
+    }
+    return swHashMap_find(SwooleG.functions, name, length);
+}
+
 #ifdef HAVE_EXECINFO
 void swoole_print_trace(void)
 {
