@@ -16,7 +16,6 @@
 
 #include "php_swoole.h"
 #include "socks5.h"
-#include "module.h"
 
 #include "ext/standard/basic_functions.h"
 
@@ -207,6 +206,7 @@ ZEND_END_ARG_INFO()
 ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_client_sendfile, 0, 0, 1)
     ZEND_ARG_INFO(0, filename)
     ZEND_ARG_INFO(0, offset)
+    ZEND_ARG_INFO(0, length)
 ZEND_END_ARG_INFO()
 
 ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_client_sendto, 0, 0, 3)
@@ -533,7 +533,7 @@ void php_swoole_client_check_setting(swClient *cli, zval *zset TSRMLS_DC)
         {
             if (Z_TYPE_P(v) == IS_STRING)
             {
-                swProtocol_length_function func = swModule_get_global_function(Z_STRVAL_P(v), Z_STRLEN_P(v));
+                swProtocol_length_function func = swoole_get_function(Z_STRVAL_P(v), Z_STRLEN_P(v));
                 if (func != NULL)
                 {
                     cli->protocol.get_package_length = func;
@@ -1124,6 +1124,11 @@ static PHP_METHOD(swoole_client, connect)
             RETURN_FALSE;
         }
 
+        if (timeout > 0)
+        {
+            php_swoole_check_timer((int) (timeout * 1000));
+        }
+
         if (swSocket_is_stream(cli->type))
         {
             if (!cb->onConnect)
@@ -1282,8 +1287,9 @@ static PHP_METHOD(swoole_client, sendfile)
     char *file;
     zend_size_t file_len;
     long offset = 0;
+    long length = 0;
 
-    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|l", &file, &file_len, &offset) == FAILURE)
+    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|ll", &file, &file_len, &offset, &length) == FAILURE)
     {
         return;
     }
@@ -1306,7 +1312,7 @@ static PHP_METHOD(swoole_client, sendfile)
     }
     //clear errno
     SwooleG.error = 0;
-    int ret = cli->sendfile(cli, file, offset);
+    int ret = cli->sendfile(cli, file, offset, length);
     if (ret < 0)
     {
         SwooleG.error = errno;
