@@ -397,11 +397,6 @@ static PHP_METHOD(swoole_websocket_server, push)
     {
         RETURN_FALSE;
     }
-    else if (length == 0)
-    {
-        php_error_docref(NULL TSRMLS_CC, E_WARNING, "data is empty.");
-        RETURN_FALSE;
-    }
 
     swConnection *conn = swWorker_get_connection(SwooleG.serv, fd);
     if (!conn || conn->websocket_status < WEBSOCKET_STATUS_HANDSHAKE)
